/****************************************/
/*		 Trabajo Fin de Master			*/
/* 		Control_trayectoria.c			*/
/* 		Author: John Vera   			*/
/* Date: 06/12/2016      				*/
/*Control trayectoria brazo robot      */
/****************************************/
#include "DSP2802x_Device.h"     		// DSP2802x Headerfile Include File
#include "PeripheralHeaderIncludes.h" 	// Esta funcion contiene todas las definiciones para configurar los relojes de los perifericos asi como las interrupciones
#include "DSP2802x_Examples.h"   		// Configura el reloj del dsc por medio del PLL
#include <string.h>				 		// Funcion que define todas las funciones para usarlas en la construccion y operacin con cadenas de caracteres
#include <stdio.h>						// Archivo de cabecera de E/S que incluye las definiciones de las macros, las constantes y declaraciones de funciones de la biblioteca standar de C
#include <stdlib.h>						// Archivo de cabecera que incluye las funciones prototipo para gestin de la memoria dinmica, control de procesos, entre otras
#include <math.h>						// Script de cabecera para incluir funciones matematicas
#include <complex.h>
#include "GLOBAL_DEFS.h"				// Definiciones globales lo del reloj del DSC de los perifericos
#include "GLOBAL_VARS.h"				// Variables globales contadores de los eventos de loa timers, de los SCI, interrupciones y banderas
#include "GLOBAL_FUNCS.h"				// Funciones globales de inicializacion de perifericos, interrupciones y timers

#define PI   3.1415926535897932384626433832795    // Se define al valor PI
#define PRG_FLASH 1				// Define la constante PRG_FLASH = 1
#define CR 13//0x0D				// Carriage Return
#define LF 10//0x0A				// Line Feed

//////////////////////////////////////////////////////////////////////////////////////
// FUNCIONES PARA LA TRANSMISION Y RECEPCION
void scia_xmit(int a);		// Llama a la funcion que transmite caracter por caracter por el periferico SCI
void scia_msg(char *msg);	// La funcion envia caracter por caracter del mensaje almacenado hasta el valor NULL
void recepcion();			// LLama a la funcin que almacena los ngulos y posiciones en las variables correspondientes
void limpiar_buffer();
void enviar(int env);
// FUNCION DE RETARD
void temporizador();

// VARIABLES PARA LA COMUNICACION Y RECEPCION DE LOS DATOS
char *msg;	// Puntero del mensaje
char delim[]=","; // Variable delimitadora para separar a los datos recibidos
char array_buffer[37];	// Vector que almacena los datos recibidos en serie por el robot
int n=0,m=0,k=0;		// Variables de uso general para las iteraciones en las funciones y programa principal
//////////////////////////////////////////////////////////////////////////////////////
// TABLA DE PARAMETROS DE DENAVIT HARTENBERG
float TablaDH[5][6]={ {0,-90,200,180,1,0},{250,0,0,180,2,0},
   {300,0,0,100,3,0},{0,-90,0,80,4,0},{0,0,150,180,5,0} };

// TIEMPO PARA LA RESOLUCION EN EL CONTROL CINEMATICO
int tiempo=2;

// CONSTANTES DE LOS COMANDOS PARA ENVIO DISPUESTOS EN X+,X-,Y+,Y-,Z+,Z-,TH4up,TH4dn,TH5r,TH5l,Fo,Fc
// COMANDOS ENVIADOS SON: D, A, Q, E, W, S, Z, X, C, V, M, N en su respectivo orden
int xp=68,xn=65,yp=87,yn=83,zp=69,zn=81,th4p=90,th4n=88,th5r=67,th5l=86,fo=77,fc=78;
//int movimientos[12]={68,65,87,83,69,81,90,88,67,86,77,78};  //DEC
//int movimientos[12]={0x44,0x41,0x57,0x53,0x45,0x51,0x5A,0x58,0x43,0x56,0x4D,0x4E};  //HEX

// VARIABLES PARA LOS ANGULOS Y POSICION DE LOS EJES DEL BRAZO ROBOT
float ang_giro=0, ang_brazo=0, ang_anteb=0, ang_cabeceo=0, ang_rotacion=0, dist_dedos=0, pos_x=0, pos_y=0, pos_z=0;

// VARIABLES PARA EL CALCULO DE LA SINGULARIDAD
float det_j, Jdet, val_J;
int th1, th2, th3, th4, th5, th6, x, y;
int a1, a2, a3, a4, a5, al1, al2, al3, al4, al5, d1, d2, d3, d4, d5;

// VARIABLES PARA EL CALCULO DEL CONTROL CINEMATICO
int a, b, c, d, e, f, g, i ,j, q;

// VARIABLES PARA EL CALCULO DE LA CINEMATICA DIRECTA
float Ai[4][4][6],A[4][4];
float temp[4][4];
int direc[4][4][5];
int id,idd,jd,jdd,kd;
float ad,alfa,theta,dd;
float rad=PI/180.0; // convierte de grados a radianes -> cos,sin,tan

// VARIABLES PARA EL CALCULO DE LA CINEMATICA INVERSA
float px, py, pz, pitch, roll, r, fx, lx, fy, ly, l, alfaa, beta, gam, cin_inv[5], auxgam, auxgamd;
float grad=180.0/PI; // convierte de radianes a grados -> acos, asin,atan

// VARIABLES GLOBALES PARA LA POSICION FINAL DADA POR EL USUARIO
float array_pff[3][5]={{0,0,899,90,0},{699,0,200,0,0},{0,-550,200,45,0}};
float pff[5];	// Variable que almacena las posiciones XYZ, cabeceo y rotacin
float pff_aux[5];	// Variable que almacena las posiciones XYZ, cabeceo y rotacin para la comparacin en el programa principal

// VARIABLES PARA LOS ANGULOS INICIALES Y FINALES DEL ROBOT
float qii[5];		//={0,-81.1426,110.4873,330.6553,180.0000};
float qff[5];       //{180,90,0,270,0};
int tray[20][5];	// Variable que almacena la trayectoria generado por el control cinematico

// VARIABLES GLOBAES
int contsing=0, flag_inv=0, flag_rx=0, flag_ctrl=0, posx=0, posy=0, posz=0, flag_ret=0, flag_eq=0;

// VARIABLE PARA LA FUNCION DE RETARDO
unsigned long contwait=0;

//////////////////////////////////////////////////////////////////////////////////////
// FUNCIONES PARA REALIZAR EL CONTROL DE TRAYECTORIA DE UN BRAZO ROBOT
int singularidad(float q[5]);	// Llama a la funcin para obtener el determinante del Jacobiano para determinar la singularidad en el espacio de trabajo
int CTRLcinematico(int qi,int qf, float t,int g);	// Llama a la funcin para obtener la trayectoria que debe realizar el brazo robot
void Cinedirecta(float robot[5][6], int N, float thetas[5]);	// Llama a la funcin para determinar la cinemtica directa
void cine_inv(float position[5]);	// Llama a la funcin para determinar la cinemtica inversa

void main(void) {
	DeviceInit();		// Inicializacin del dispositivo
	InitDSP();			// Inicializacion de perifericos


	#if (PRG_FLASH==0)
		MemCopy(&RamfuncsLoadStart, &RamfuncsLoadEnd, &RamfuncsRunStart);//copia a la memoria flash
		InitFlash();	// Call the flash wrapper init function
	#endif //(FLASH)
///////////////////////////////////////////////////////////////////////////////////////
//	ENVIA CARACTERES PARA COMENZAR LA COMUNICACION CON EL ROBOT
	enviar(xp);	// Envia el caracter
	enviar(xn);
//	LIMPIA EL BUFFER DE RECEPCION
	limpiar_buffer();
//	BUCLE SIN FIN
	for(;;){
		while(VTimer0 >= 1){
			VTimer0 = 0;
			EALLOW;
				SysCtrlRegs.WDKEY = 0x55;	// service WD #1
				SysCtrlRegs.WDKEY = 0xAA;
			EDIS;
			if(flag_ret==0){
				if(k<3){
					pff[0]=array_pff[k][0];
					pff[1]=array_pff[k][1];
					pff[2]=array_pff[k][2];
					pff[3]=array_pff[k][3];
					pff[4]=array_pff[k][4];
					flag_ret=1;
				}
			}
			if(flag_rx==1){
				if(flag_ctrl==0){
					// PROCESO QUE LLAMA A LA FUNCION CINEMATICA INVERSA
					// PARA DE UNA POSICION DADA DEVUELVA LOS ANGULOS DE CADA EJE DEL ROBOT
					if(flag_inv==0){
						cine_inv(pff);
						flag_inv=1;
						for(i=0;i<6;i++){
							qff[i]=cin_inv[i];
						}
					}
					// PROCESO QUE OBTIENE LA TRAYECTORIA A SEGUIR
					// ESTE PROCESO ES EL CONTROL CINEMATICO
					for (i=0;i<(tiempo*10);i=i+1){
						for (j=0;j<5;j++){
							tray[i][j]=CTRLcinematico(qii[j],qff[j],tiempo,i);
						}
					}
					// PROCESO QUE OBTIENE LA SINGULARIDAD DE LA POSICION FINAL
					// EN EL CASO QUE TENGA UNA SINGULARIDAD PERSISTENTE EL PROGRAMA SE TERMINA
					det_j=singularidad(qff);
					val_J=cimag(det_j);
							if(val_J!=0){
								//contsing=contsing+1;
								//if(contsing==2){
									exit(0);
								//}
							}
							else{
								contsing=0;
							}
							if ((pff[0]>700)||(pff[0]<-700)||(pff[1]>700)||(pff[1]<-700)||(pff[2]>900)||(pff[2]<0)){
								exit(0);
							}
					// FUNCION QUE REALIZA LA CINEMATICA DIRECTA
					// DESPUES DEL ANALISIS PREVIO PARA COMPROBAR QUE ESA POSICION FINAL ES VALIDA
					Cinedirecta(TablaDH,5,qff);
					flag_ctrl=1;
					if(flag_eq==0){
						posx=pos_x;
						posy=pos_y;
						posz=pos_z+200;
						flag_eq=1;
					}
				}
				if(posx<(int)pff_aux[0]){
					enviar(xp);
					posx++;
				}
				if(posx>(int)pff_aux[0]){
					enviar(xn);
					posx--;
				}
				if(posy<(int)pff_aux[1]){
					enviar(yp);
					posy++;
				}
				if(posy>(int)pff_aux[1]){
					enviar(yn);
					posy--;
				}
				if(posz<(int)pff_aux[2]){
					enviar(zp);
					posz++;
				}
				if(posz>(int)pff_aux[2]){
					enviar(zn);
					posz--;
				}
				if(ang_cabeceo<((int)pff_aux[3]-90)){
					enviar(th4p);

				}
				if(ang_cabeceo>((int)pff_aux[3]-90)){
					enviar(th4n);
				}
				if(((pos_x==(int)pff_aux[0])&&(pos_y==(int)pff_aux[1])&&(posz==(int)pff_aux[2]))||((posx==(int)pff_aux[0])&&(posy==(int)pff_aux[1])&&(posz==(int)pff_aux[2]))){
					flag_ret=0;
					flag_ctrl=0;
					flag_inv=0;
					enviar(xp);
					enviar(xn);
					k++;

				}
				if(k==3){exit(0);}

				//temporizador();

			}
		}
	}

}

//////////////////////////////////////////////////////////////////////////////////////
// FUNCION QUE LIMPIA EL VECTOR GLOBAL QUE ALMACENA LOS DATOS DE LA RECEPCION
void limpiar_buffer(){
	for(i=0;i<38;i++){
			array_buffer[i]=NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////////////
// FUNCION DE RETARDO
void temporizador(){
	while(contwait++<50000){
		asm(" NOP"); // es un retardo de software 16.66667ns => t=1/f=1/60mhz
		EALLOW;
		SysCtrlRegs.WDKEY = 0x55; // Habilita Watchdog llave de reseteo al registro para la escritura
		SysCtrlRegs.WDKEY = 0xAA; // Watchdog llave de reseteo al registro pone al contador en 0
		EDIS;
	}
	contwait=0;
}

//////////////////////////////////////////////////////////////////////////////////////
// 	INTERRUPCIN QUE ACTIVA EL TIMER
interrupt void TA0_Cpu_Timer0_isr(void){
	VTimer0++;
	EALLOW;
		SysCtrlRegs.WDKEY = 0xAA;	// service WD #2
	EDIS;
	PieCtrlRegs.PIEACK.all = PIEACK_GROUP1;
}

//////////////////////////////////////////////////////////////////////////////////////
// 	FUNCIONES PARA ENVIO DE LOS VALORES ASCII EN DEC
/*
  @brief Con esta funcin se envian los valores DEC para que el brazo robot pueda
  moverse segun el codigo en el vector
*/
// FUNCION QUE ENVIA UN CARACTER POR COMUNICACION SERIE
void enviar(int env){
	scia_xmit(env);
	scia_xmit(CR);
	scia_xmit(LF);

}

//////////////////////////////////////////////////////////////////////////////////////
// FUNCION QUE SEPARA LOS DATOS EN LOS DIFERENTES ANGULOS Y POSICION
void recepcion(){

	msg=strtok(array_buffer, delim); // STRTOK FUNCION QUE SEPARA CADENAS SEGUN EL DELIMITADOR
	ang_giro = atoi(msg)-1000;  // ATOI FUNCION QUE CONVIERTE UNA CADENA EN ENTERO
	qii[0]=ang_giro+1000;
	msg=strtok(NULL, delim);
	ang_brazo = atoi(msg);
	qii[1]=ang_brazo;
	msg=strtok(NULL, delim);
	ang_anteb = atoi(msg);
	qii[2]=ang_anteb;
	msg=strtok(NULL, delim);
	ang_cabeceo = atoi(msg);
	qii[3]=ang_cabeceo;
	msg=strtok(NULL, delim);
	ang_rotacion = atoi(msg);
	qii[4]=ang_rotacion;
	msg=strtok(NULL, delim);
	dist_dedos = atoi(msg);
	msg=strtok(NULL, delim);
	pos_x = atoi(msg);
	msg=strtok(NULL, delim);
	pos_y = atoi(msg);
	msg=strtok(NULL, delim);
	pos_z = atoi(msg);
}

//////////////////////////////////////////////////////////////////////////////////////
// FUNCION DE LA CINEMATICA INVERSA
/**
  @brief Funcion que devuelve los ngulos de cada articulacin del brazo robot de la posicin
  cabeceo y rotacin ingresados de la mueca o efector final, datos del punto a
 */
void cine_inv(float position[5]){
	float lh = 200; // Longitud del hombro
	float lb = 250; // Longitud del brazo
	float la = 300; // Longitud del antebrazo
	float lm = 150; // Longitud de la mueca + herramienta
	px = position[0]; // Posicin en el eje X
	py = position[1]; // Posicin en el eje Y
	pz = position[2]; // Posicin en el eje Z
	pitch = position[3]; // Cabeceo de la mueca
	roll= position[4]; // Rotacin de la mueca

	// Calculo Theta 1
	if(px!= 0){
		cin_inv[0] = atan(py/px)*grad;
		if((cin_inv[0]<=-150)&&(cin_inv[0]>=150)){
			if(cin_inv[0]<-150){
				cin_inv[0] = -150;
			}
			else{
				cin_inv[0] = 150;
			}
		}
	}
	else{
		cin_inv[0] = atan(py/px)*grad;
	}
	if((px == 0)&&(py == 0)){
		cin_inv[0] = 180;
	}
	if((px!=0)&&(py!=0)){
		r = sqrt(abs(pow(px,2))+abs(pow(py,2)));
	}
	if(py==0){
		r=abs(px);
	}
	if(px==0){
		r=abs(py);
	}
	// Calculo Theta 2
	fx = lm*cos(pitch*rad);
	lx = r-fx;
	fy = lm*sin(pitch*rad);
	ly = pz-fy-lh;
	//if((lx!=0)&&(ly!=0)){
		l = sqrt((pow(lx,2))+(pow(ly,2)));
	//}
	if(ly==0){
		l=lx;
	}
	if(lx==0){
		l=ly;
	}
	alfaa = atan(ly/lx)*grad;
	beta = acosf(((pow(lb,2)+pow(l,2)-pow(la,2))/(2*lb*l)))*grad;
	cin_inv[1] = (-(alfaa+beta));
	if((px == 0)&&(py == 0)){
		cin_inv[1] = -90;
	}

	// Calculo Theta 3
	auxgam=pow(lb,2)+pow(la,2)-pow(l,2);
	auxgamd=2*(lb*la);
	gam = acos(auxgam/auxgamd)*grad;
	cin_inv[2] = (180-gam);
	if((px == 0)&&(py == 0)){
		cin_inv[2] = 0;
	}

	// Calculo Theta 4
	cin_inv[3] = 360-90+(-pitch-cin_inv[1]-cin_inv[2]);
	//cin_inv[3] = pitch-cin_inv[1]-cin_inv[2];

	cin_inv[4] = roll;
	// Theta 5 ingresa el usuario
}

//////////////////////////////////////////////////////////////////////////////////////
// FUNCION DE LA CINEMATICA DIRECTA
/**
  @brief Funcion que devuelve la posicin X, Y y Z de los ngulos de cada articulacin
  del brazo robot de su punto final, usando la tabla de Denavit Hartenberg y matrices
  homogneas. Para a posterior realizar las comparaciones para realizar las otras
  funciones.
 */
void Cinedirecta(float robot[5][6], int N, float thetas[5]){
	// Limpia la matriz temporal
	for (id=0;id<4;id++){		// Bucle FOR para ingresar en la matriz temp e igualarla a cero
		for (jd=0;jd<4;jd++){
			temp[id][jd]=0;		// Matriz temporal para almacenar la matriz homogenea
		}
	}
	// Crea una matriz unitaria
	for (id=0;id<4;id++){
		for (jd=0;jd<4;jd++){
			if (id != jd){
				Ai[id][jd][0] = 0;
			}
			else{
				Ai[id][jd][0] = 1;
			}
		}
	}
	// Se obtiene los valores de interes de la tabla de Denavit Hartenberg
	for (id=1;id<N+1;id++){
	    ad=robot[id-1][0];
	    alfa=robot[id-1][1];
	    //phi = 0 es revolucion phi = 1 es prismatico
	    if (robot[id-1][5] == 0){
	        theta=thetas[id-1];
	        dd=robot[id-1][2];
	    }
	    else{
	        dd=thetas[id-1];
	        theta=robot[id-1][3];
	    }
	    // Matriz homognea DH clasica
	    A[0][0]=cos(theta*rad); A[0][1]=-cos(alfa*rad)*sin(theta*rad); A[0][2]=sin(alfa*rad)*sin(theta*rad); A[0][3]=ad*cos(theta*rad);
	    A[1][0]=sin(theta*rad); A[1][1]=cos(alfa*rad)*cos(theta*rad); A[1][2]=-sin(alfa*rad)*cos(theta*rad); A[1][3]=ad*sin(theta*rad);
	    A[2][0]=0; A[2][1]=sin(alfa*rad); A[2][2]=cos(alfa*rad); A[2][3]=dd;
	    A[3][0]=0; A[3][1]=0; A[3][2]=0; A[3][3]=1;

	    // Multiplicacin de matrices inicio
	    for (idd = 0; idd < 4; idd++){
	        for (jdd = 0; jdd < 4; jdd++){
	        	temp[idd][jdd]=0;
	            for (kd = 0; kd < 4; kd++){
	                temp[idd][jdd] += Ai[idd][kd][id-1] * A[kd][jdd];
	            }
	        }
	    }
	    // Multiplicacin de matrices fin
	    // Almacenado de las matrices homogeneas en la matrz auxiliar
	    for (idd = 0; idd < 4; idd++) {
	        for (jdd = 0; jdd < 4; jdd++) {
	        	  Ai[idd][jdd][id]=temp[idd][jdd];
	        }
	    }
	}
	// Almacena en la matriz homognea final por cada eje
	for (id=0;id<N;id++){
		for (idd=0;idd<4;idd++){
			for (jdd = 0; jdd < 4; jdd++) {
				direc[idd][jdd][id]=Ai[idd][jdd][id+1];
			}
		}
	}
	pff_aux[0]=direc[0][3][4];
	pff_aux[1]=direc[1][3][4];
	pff_aux[2]=direc[2][3][4];
	pff_aux[3]=qff[3];
	pff_aux[4]=qff[4];

}

//////////////////////////////////////////////////////////////////////////////////////
// FUNCION DEL CONTROL CINEMATICO POR MEDIO DE UN POLINOMIO DE 5TH ORDEN
/**
  @brief Funcion que mediante los 5 thetas de la posicion inicial
  y con los 5 thetas de la posicion final mas un tiempo de ejecucion
  se obtiene una matriz de angulos para cada theta o eje del brazo robot
  necesarios para sumar o restar los thetas iniciales hasta los thetas finales.
 */
int CTRLcinematico(int qi,int qf, float t, int g){
   a=(qi*pow(t,3)*pow(t,2)/pow(t,5));
   b=0;
   c=0;
   d=(qf*(10*pow(t,2))/pow(t,5))-(qi*(10*pow(t,2))/pow(t,5));
   e=(qi*(15*t)/pow(t,5))-(qf*(15*t)/pow(t,5));
   f=(6*qf/pow(t,5))-(6*qi/pow(t,5));
   q=(f*pow(g/10.0,5) + e*pow(g/10.0,4) + d*pow(g/10.0,3) + c*pow(g/10.0,2) + b*g + a);
   //printf("%d ",i);
   return q;
 // printf("\n");
}

//////////////////////////////////////////////////////////////////////////////////////
// FUNCION PARA EL CALCULO DE LA SINGULARIDAD DEL BRAZO ROBOT
/**
  @brief Funcion que mediante los 5 thetas de la posicion final determinara
  la singularidad del sistema mediente el determinante del Jacobiano si da
  cero dara aviso por medio de una bandera para que en una segunda vez realice
  la funcion break en el cuerpo principal y termine el programa.
 */
int singularidad(float qs[5]){
	th1=qs[0];
	th2=qs[1];
	th3=qs[2];
	th4=qs[3];
	th5=qs[4];

	a1=0; a4=0; a5=0; al1=-PI/2; al2=0; al3=0; al4=-PI/2; al5=0; d2=0; d3=0;
	d4=0; th6=th5; a2=250; a3=300; d1=200; d5=150;
	Jdet=-(a2*a3*(a3*cos(th3+th4)-a3*cos(2*th2+th3+th4)-d5*sin(2*th2+3*th3+2*th4)-a2*cos(th4)+a2*cos(2*th2+2*th3+th4)+a3*cos(2*th2+3*th3+th4)+2*d5*sin(th3)+d5*sin(2*th2+th3+2*th4)-a2*cos(2*th2+th4)+a2*cos(2*th3+th4)-a3*cos(th3-th4)))/4;
	return(Jdet);
}

//////////////////////////////////////////////////////////////////////////////////////
// 	INICIALIZACIN DE LOS PERIFERICOS DEL DSP
/**
  @brief Con esta funcin se inicializan los perifericos del DSC TMS320F28027

    InitTimers();	// Inicializacin de los temporizadores
	InitADC();	// Inicializacin del conversor analgico - digital
	InitPWM();	// Inicializacion del mdulo PWM 1 y 2
	InitSci();	// Inicializamos el modulo SCI
	InitInt();	// Inicializacin de las interrupciones
	@return NO se devuelve nada
*/
void InitDSP(){
	// Deshabilitamos las interrupciones de la CPU
   	DINT;
	// Deshabilitamos las interrupciones de la CPU y borramos los flags
   	IER = 0x0000;
   	IFR = 0x0000;
	InitSci();
	InitInt();
	InitTimers();	// Inicializacin de los temporizadores
}

//////////////////////////////////////////////////////////////////////////////////////
//	INTERRUPCIN DE RECEPCIN DEL BUS SCI-A
/**
  @brief Funcin de interrupcin de recepcin (RX) del SCI-A
  Se produce esta interrupcin cada vez que se reciben 4 bytes desde el
  bus SCI-A
  //////////////////////////////////////////////////////////////////
  // PETICION desde el PC-Ordenador ==> DSC ....
  // TRAMA: 0x05 - ORDEN - MENSAJE - CHKSUM
  // ORDEN: 0x51 ==> Envio_Tension
  // ORDEN: 0x52 ==> Envio_Frecuencia
  // ORDEN: 0x53 ==> Recepcion_Angulo
  ///////////////////////////////////////////////////////////////
  Utiliza los valores de los arrays: trama_RX[] y trama_TX[]
  @return NO se devuelve nada
*/
interrupt void SCIA_RXFIFO_isr(void){
	SciRX_Count++;
	// Almacenar valores de la TRAMA....
	array_buffer[n] = SciaRegs.SCIRXBUF.all;
	if((array_buffer[n-1]==CR)&&(array_buffer[n]==LF)){
		recepcion();
		flag_rx=1;
		n=0;
	}
	n++;
	////////////////////////////////////////////////////
	// Decodificar o interpretar la ORDEN de la TRAMA....

	// Restaurar Flags para siguiente interrupcin.....
	SciaRegs.SCIFFRX.bit.RXFFOVRCLR=1;   // Clear Overflow flag
	SciaRegs.SCIFFRX.bit.RXFFINTCLR=1;   // Clear Interrupt flag
	PieCtrlRegs.PIEACK.all = PIEACK_GROUP9;       // Issue PIE ack
}

//////////////////////////////////////////////////////////////////////////////////////
// 	INICIALIZACIN DE LOS TEMPORIZADORES
/**
  @brief Funcin de inicializacin de los Temporizadores del 28027

  Inicializacin de los 3 timers del DSC.
  Se configuran de la siguiente manera:
	Timer0:	100ms
	Timer1:	  1ms
	Timer2:	100us

*/
void InitTimers (void){

	EALLOW;
		SysCtrlRegs.PCLKCR3.bit.CPUTIMER0ENCLK = 1; // CPU Timer-0
   		//SysCtrlRegs.PCLKCR3.bit.CPUTIMER1ENCLK = 1; // CPU Timer-1
   		//SysCtrlRegs.PCLKCR3.bit.CPUTIMER2ENCLK = 1; // CPU Timer-2
   	EDIS;

	// Asignamos los manejadores de interrupcin a los timers
	EALLOW;
	   	PieVectTable.TINT0 = &TA0_Cpu_Timer0_isr;
		//PieVectTable.TINT1 = &TB0_Cpu_Timer1_isr;
		//PieVectTable.TINT2 = &TC0_Cpu_Timer2_isr;
	EDIS;
	// Inicializamos los perifericos de los temporizadores
   	InitCpuTimers();   // For this example, only initialize the Cpu Timers

	// Configuramos los temporizadores
	ConfigCpuTimer(&CpuTimer0, 60, 150000);		//100ms
	//ConfigCpuTimer(&CpuTimer1, 60, 1000);		//1ms
	//ConfigCpuTimer(&CpuTimer2, 60, 100);		//0.1ms


  	CpuTimer0Regs.TCR.all = 0x4001; // Use write-only instruction to set TSS bit = 0
   //	CpuTimer1Regs.TCR.all = 0x4001; // Use write-only instruction to set TSS bit = 0
   //	CpuTimer2Regs.TCR.all = 0x4001; // Use write-only instruction to set TSS bit = 0


	/*************************************************************************************/

	VTimer0 = 0;	// Contador del Evento del Timer-0
	//VTimer1 = 0; 	// Contador del Evento del Timer-1
	//VTimer2 = 0; 	// Contador del Evento del Timer-2
}

//////////////////////////////////////////////////////////////////////////////////////
//	INCIALIZACIN DEL MODULO SCI
/**
  @brief Funcin de inicializacin del bus SCIA

  Inicializacin del mdulo de comunicaciones SCI-A. En el caso de la placa
  launchpad C2000, este mdulo se corresponde con la comunicacin USB, que
  genera en el PC un puerto COM mediante el cual nos podemos comunicar como
  un UART.
  Parametros de comunicacion: 9600, 8, N; 1
  Habilitamos la FIFO...
  Configuramos para que cuando haya 4 bytes en la FIFO provoque una interrupcion (RX)
*/
void InitSci(void) {

   	EALLOW;
		GpioCtrlRegs.GPAPUD.bit.GPIO28 = 0;    // Habilitamos resistencia de PULL-UP (SCIRXDA)
		GpioCtrlRegs.GPAPUD.bit.GPIO29 = 1;	   // Deshabilitamos resistencia de PULL-UP (SCITXDA)

		GpioCtrlRegs.GPAQSEL2.bit.GPIO28 = 3;  // Entrada asincrona en GPIO28 (SCIRXDA)

		GpioCtrlRegs.GPAMUX2.bit.GPIO28 = 1;   // Configuramos el puerto como SCIRXDA
		GpioCtrlRegs.GPAMUX2.bit.GPIO29 = 1;   //Configuramos el puerto como SCITXDA

		SysCtrlRegs.PCLKCR0.bit.SCIAENCLK = 1; // Habilitamos el reloj para SCI-A
    EDIS;

	SciaRegs.SCICCR.bit.SCICHAR = 7;	// 8 bits de datos
	SciaRegs.SCICTL1.bit.RXENA = 1;		// Habilitamos la recepcin
	SciaRegs.SCICTL1.bit.TXENA = 1;		// Habilitamos la transmisin

	SciaRegs.SCICTL2.bit.RXBKINTENA = 1;	// Habilitamos la interrupcin RXRDY
	SciaRegs.SCICTL2.bit.TXINTENA = 1;		// Habilitamos la interrupcin TXRDY

	SciaRegs.SCIHBAUD = 0;
	//SciaRegs.SCILBAUD = 194;		//BaudRate = 9600
	SciaRegs.SCILBAUD = 97;		//BaudRate = 19200
	//SciaRegs.SCILBAUD = 48;		//BaudRate = 38400
	//SciaRegs.SCILBAUD = 32;		//BaudRate = 57600
	//SciaRegs.SCILBAUD = 15;		//BaudRate = 115200

	SciaRegs.SCIFFTX.bit.SCIFFENA = 1;	//Habilitamos la FIFO
	SciaRegs.SCIFFRX.bit.RXFFIENA = 1;	//Habilitamos la interrupcin de la fifo de recepcin
	SciaRegs.SCIFFRX.bit.RXFFIL = 4;	//La FIFO solo tiene 4 niveles

	EALLOW;
	   	PieVectTable.SCIRXINTA = &SCIA_RXFIFO_isr;
	EDIS;

	SciaRegs.SCICTL1.bit.SWRESET = 1;	// Salimos del reset

}

////////////////////////////////////////////////////////////////////////////////////
// ENVIA UN CARACTER POR SCI
void scia_xmit(int a){
    while (SciaRegs.SCIFFTX.bit.TXFFST != 0) {} // Espera hasta que se reciba el dato
    SciaRegs.SCITXBUF=a;

}

void scia_msg(char * msg){ // La funcion envia caracter por caracter del mensaje almacenado hasta el valor NULL o 0\//
    int i;
    i = 0;
    while(msg[i] != '\0')
    {
        scia_xmit(msg[i]);
        i++;
    }
}

//////////////////////////////////////////////////////////////////////////////////////
// 	INICIALIZACIN DE INTERRUPCIONES
/**
  @brief Funcin de inicializacin las interrupciones.

  Se habilitan e inicializan todas las interrupciones necesarias.
*/
void InitInt(void){
	 //Enable Peripheral, global Ints and higher priority real-time debug events:
   	IER |= (M_INT1|M_INT2);
   	IER |= (M_INT3|M_INT4);
   	IER |= M_INT8;
   	IER |= M_INT9; // Esta deja Bloqueado al HW...
	// Activar interrupcin sci fifo rx
	PieCtrlRegs.PIEIER9.bit.INTx1 = 1;   // RX SCIA
	PieCtrlRegs.PIEIER1.bit.INTx7 = 1;	 // Activar Interrupcin T0

	// Enable the PIE
    PieCtrlRegs.PIECTRL.bit.ENPIE = 1;
	// Enables PIE to drive a pulse into the CPU
	PieCtrlRegs.PIEACK.all = 0xFFFF;
	// Enable Interrupts at the CPU level
    EINT;   // Enable Global interrupt INTM
	ERTM;   // Enable Global realtime interrupt DBGM
	//////////////////////////////////////////////////////
}
